/***************************************************************************
 *   Copyright  2009 - 2010 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "ActionsBox.h"
#include "potd.h"

// Qt
#include <QGraphicsLinearLayout>
#include <QGraphicsAnchorLayout>
#include <QtCore/QTimer>

#include <QDebug>

// KDE
#include <KIcon>
#include <KIconLoader>
#include <KPushButton>

// Plasma
#include <Plasma/IconWidget>
#include <Plasma/Animation>

#define cAppearance                 500
#define cFadeInDuration             250
#define cFadeOutDuration            500

using namespace Plasma;

QHash<quint64, ActionsBox*> *ActionsBox::_instances = new QHash<quint64, ActionsBox*>();

ActionsBox* ActionsBox::Instance( ActionBoxPosition boxPos, Applet *parent )
{
    // we use pointer as a key
    quint64 key = (quint64)parent;

    ActionsBox *inst = _instances->value( key, 0 );

    if( 0 == inst )
    {
        // this is new insance of PhotoOfTheDay so
        // we need new instance of ActionsBox class

        inst = new ActionsBox( boxPos, parent );

        _instances->insert(key, inst);
    }

    return inst;
}

ActionsBox::ActionsBox(  ActionBoxPosition boxPos, Applet *parent ):Frame( parent )
{
    m_state = StateCount;
    m_parent = parent;

    setFrameShadow( Raised );
    setZValue( 10 );
    setOpacity(0.0);

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout( Qt::Horizontal, this );
    layout->setSpacing( 5.0 );

    m_prevEngine = new PushButton(this);
    m_prevEngine->nativeWidget()->setIcon( KIcon("media-seek-backward") );
    m_prevEngine->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    m_prevEngine->setMaximumSize( IconSize( KIconLoader::MainToolbar ), IconSize( KIconLoader::MainToolbar ) );
    connect(m_prevEngine, SIGNAL(clicked()), this, SLOT(showPrevEngine()));
    layout->addItem( m_prevEngine );

    m_nextEngine = new PushButton(this);
    m_nextEngine->nativeWidget()->setIcon( KIcon("media-seek-forward") );
    m_nextEngine->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    m_nextEngine->setMaximumSize( IconSize( KIconLoader::MainToolbar ), IconSize( KIconLoader::MainToolbar ) );
    connect(m_nextEngine, SIGNAL(clicked()), this, SLOT(showNextEngine()));
    layout->addItem( m_nextEngine );

//    m_save = new IconWidget(this);
//    m_save->setPreferredSize(22.0, 22.0);
//    m_save->setIcon( KIcon("document-save") );
//    layout->addItem( m_save );
//    connect(m_save, SIGNAL(clicked()), this, SLOT(savePhoto()));

    m_open = new PushButton(this);
    m_open->nativeWidget()->setIcon( KIcon( "download" ) );
    m_open->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    m_open->setMaximumSize( IconSize( KIconLoader::MainToolbar ), IconSize( KIconLoader::MainToolbar ) );
    connect(m_open, SIGNAL(clicked()), this, SLOT(openWebsite()));
    layout->addItem( m_open );

    setLayout(layout);

    // appearance timer
    m_appearaceTimer = new QTimer( this );
    m_appearaceTimer->setSingleShot( true );
    m_appearaceTimer->setInterval( cAppearance );
    connect( m_appearaceTimer, SIGNAL(timeout()), this, SLOT(appearTimeout()) );

    // initialize animations
    // start animation
    m_fadeIn = Plasma::Animator::create( Plasma::Animator::FadeAnimation );
    m_fadeIn->setProperty( "startOpacity", 0.0 );
    m_fadeIn->setProperty( "targetOpacity", 1.0 );
    m_fadeIn->setProperty( "duration", cFadeInDuration );
    m_fadeIn->setTargetWidget( this );
    connect(m_fadeIn, SIGNAL(finished()), this, SLOT(fadeInFinished()));

    // hide animation
    m_fadeOut = Plasma::Animator::create( Plasma::Animator::FadeAnimation );
    m_fadeOut->setProperty( "startOpacity", 1.0 );
    m_fadeOut->setProperty( "targetOpacity", 0.0 );
    m_fadeOut->setProperty( "duration", cFadeOutDuration );
    m_fadeOut->setTargetWidget( this );
    connect(m_fadeOut, SIGNAL(finished()), this, SLOT(fadeOutFinished()));

    // add actions box to main layout
    QGraphicsAnchorLayout * parentLayout = dynamic_cast<QGraphicsAnchorLayout *>(parent->layout());

    switch ( boxPos )
    {
        case TopLeft:
        {
            parentLayout->addCornerAnchors( this, Qt::TopLeftCorner, parentLayout, Qt::TopLeftCorner );

            break;
        }

        case TopCenter:
        {
            parentLayout->addAnchor( this, Qt::AnchorHorizontalCenter, parentLayout, Qt::AnchorHorizontalCenter );
            parentLayout->addAnchor( this, Qt::AnchorTop, parentLayout, Qt::AnchorTop );

            break;
        }

        case TopRight:
        {
            parentLayout->addCornerAnchors( this, Qt::TopRightCorner, parentLayout, Qt::TopRightCorner );

            break;
        }

        case BottomLeft:
        {
            parentLayout->addCornerAnchors( this, Qt::BottomLeftCorner, parentLayout, Qt::BottomLeftCorner );

            break;
        }

        case BottomCenter:
        {
            parentLayout->addAnchor( this, Qt::AnchorHorizontalCenter, parentLayout, Qt::AnchorHorizontalCenter );
            parentLayout->addAnchor( this, Qt::AnchorBottom, parentLayout, Qt::AnchorBottom );

            break;
        }

        case BottomRight:
        {
            parentLayout->addCornerAnchors( this, Qt::BottomRightCorner, parentLayout, Qt::BottomRightCorner );
            break;
        }

        default:
        {
            Q_ASSERT_X( false, "ActionsBox::ActionsBox()", "Bad position" );
        }
    }

    int offset = dynamic_cast<PhotoOfTheDay *>(parent)->calcOffset() + 2;
    parentLayout->setContentsMargins( offset, offset, offset, offset );
    
    m_state = Initialized;
}

ActionsBox::~ActionsBox()
{
    // remove from hash
    _instances->remove((quint64)m_parent);

    // in case applet is destroyed, main layout takes care about removing items
    if ( m_parent->layout() != 0 )
    {
        // remove from main layout
        for(int i = 0; i < m_parent->layout()->count(); ++i)
        {
            QGraphicsLayoutItem *item = m_parent->layout()->itemAt(i);

            if( item == this )
            {
                 m_parent->layout()->removeAt(i);

                break;
            }
        }
    }
}

void ActionsBox::exec()
{
    // this function can be called multiple times during the life of object
    // every time it should "restart" the object

    switch( m_state )
    {
        case Initialized:
        {
            m_appearaceTimer->start();
            m_state = WaitingToAppear;

            break;
        }
        case WaitingToAppear:
        {
            m_appearaceTimer->stop();
            m_appearaceTimer->start();

            break;
        }
        case FadingIn:
        {
            // just go on
            break;
        }
        case Running:
        {
            // just go on
            break;
        }
        case FadingOut:
        {
            // stop and restore
            m_fadeOut->stop();

            m_fadeIn->start();
            m_fadeIn->setCurrentTime( cFadeInDuration - m_fadeOut->currentTime() / 2 );

            m_state = FadingIn;

            break;
        }
        case Dead:
        {
            // go to Initialized
            m_fadeIn->start();

            m_state = FadingIn;

            break;
        }
        default:
        {
            // this should never happen
        }
    }
}

void ActionsBox::hideActions()
{
    // this function can be called multiple times during the life of object

    switch( m_state )
    {
        case Initialized:
        {
            m_state = Dead;

            deleteLater();

            break;
        }
        case WaitingToAppear:
        {
            m_appearaceTimer->stop();
            m_state = Dead;

            break;
        }
        case FadingIn:
        {
            m_fadeIn->stop();

            m_fadeOut->start();
            m_fadeOut->setCurrentTime( cFadeOutDuration - m_fadeIn->currentTime() * 2 );

            m_state = FadingOut;

            break;
        }
        case Running:
        {
            m_fadeOut->start();

            m_state = FadingOut;

            break;
        }
        case FadingOut:
        {
           // just go on

            break;
        }
        case Dead:
        {
           // just go on

            break;
        }
        default:
        {
            // this should never happen
        }
    }
}

void ActionsBox::kill()
{
    switch ( m_state )
    {
        case Initialized:
        {
            // just go on

            break;
        }
        case WaitingToAppear:
        {
            m_appearaceTimer->stop();

            break;
        }
        case FadingIn:
        {
            m_fadeIn->stop();

            break;
        }
        case Running:
        {
            // just go on

            break;
        }
        case FadingOut:
        {
           m_fadeOut->stop();

            break;
        }
        case Dead:
        {
           // just go on

            break;
        }
        default:
        {
            // this should never happen
        }
    }

    m_state = Dead;

    deleteLater();
}

void ActionsBox::appearTimeout()
{
    m_fadeIn->start();
    m_state = FadingIn;
}

void ActionsBox::fadeInFinished()
{
    m_state = Running;
}

void ActionsBox::fadeOutFinished()
{
    m_state = Dead;

    deleteLater();
}

void ActionsBox::openWebsite()
{
     PhotoOfTheDay *potd = qobject_cast<PhotoOfTheDay *>(m_parent);
     potd->runAssociatedApplication();
}

//void ActionsBox::savePhoto()
//{
//
//}

void ActionsBox::showNextEngine()
{
    PhotoOfTheDay *potd = qobject_cast<PhotoOfTheDay *>(m_parent);
    potd->useNextEngine();
}

void ActionsBox::showPrevEngine()
{
    PhotoOfTheDay *potd = qobject_cast<PhotoOfTheDay *>(m_parent);
    potd->usePrevEngine();
}
