/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "ErrorBox.h"

// Qt
#include <QLabel>
#include <QGraphicsLinearLayout>
#include <QGraphicsAnchorLayout>
#include <QAbstractAnimation>

// KDE
#include <KIcon>
#include <KIconLoader>

// Plasma
#include <Plasma/IconWidget>

#define cFadeInDuration             250
#define cFadeOutDuration            500

using namespace Plasma;

QHash<quint64, ErrorBox*> *ErrorBox::_instances = new QHash<quint64, ErrorBox*>();

ErrorBox* ErrorBox::Instance(Applet *parent)
{
    // we use pointer as a key
    quint64 key = (quint64)parent;

    ErrorBox *inst = _instances->value(key, 0);

    if( 0 == inst )
    {
        // this is new insance of PhotoOfTheDay so
        // we need new instance of this class

        inst = new ErrorBox(parent);

        // qDebug() << "ErrorBox::Instance() Inserting with key " << key;
        _instances->insert(key, inst);
    }

    return inst;
}

ErrorBox::ErrorBox(Applet *parent):Plasma::Frame(parent)
{
    m_state = Count;
    m_parent = parent;

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout( Qt::Horizontal, this );

    IconWidget *errIcon;
    errIcon = new IconWidget( this );
    errIcon->setIcon( KIcon( "dialog-error" ) );
    errIcon->setMinimumSize( IconSize( KIconLoader::Desktop ), IconSize( KIconLoader::Desktop ) );
    layout->addItem( errIcon );

    m_errorLabel = new Label( this );
    layout->addItem( m_errorLabel );

    QLabel *lbl = m_errorLabel->nativeWidget();
    lbl->setWordWrap( true );
    lbl->setAlignment( Qt::AlignJustify | Qt::AlignVCenter );

    setLayout(layout);

    setFrameShadow( Raised );
    setMaximumSize( parent->contentsRect().width()  * 0.75, parent->contentsRect().height() * 0.5 );

    // init lifetime timer
    m_lifetime.setInterval( 10000 ); // set lifetime to 10 sek
    m_lifetime.setSingleShot( true );
    connect( &m_lifetime, SIGNAL(timeout()), this, SLOT(lifeTimeout()) );

    // initialize animactions
    // start anmation
    m_fadeIn = Plasma::Animator::create( Plasma::Animator::FadeAnimation );
    m_fadeIn->setProperty( "startOpacity", 0.0 );
    m_fadeIn->setProperty( "targetOpacity", 1.0 );
    m_fadeIn->setProperty( "duration", cFadeInDuration );
    m_fadeIn->setTargetWidget( this );
    connect(m_fadeIn, SIGNAL(finished()), this, SLOT(fadeInFinished()));

    // hide animation
    m_fadeOut = Plasma::Animator::create( Plasma::Animator::FadeAnimation );
    m_fadeOut->setProperty( "startOpacity", 1.0 );
    m_fadeOut->setProperty( "targetOpacity", 0.0 );
    m_fadeOut->setProperty( "duration", cFadeOutDuration );
    m_fadeOut->setTargetWidget( this );
    connect(m_fadeOut, SIGNAL(finished()), this, SLOT(fadeOutFinished()));

    // add error box to main layout
    QGraphicsAnchorLayout * parentLayout = dynamic_cast<QGraphicsAnchorLayout *>(parent->layout());
    parentLayout->addAnchor( this, Qt::AnchorHorizontalCenter, parentLayout, Qt::AnchorHorizontalCenter );
    parentLayout->addAnchor( this, Qt::AnchorVerticalCenter, parentLayout, Qt::AnchorVerticalCenter );

    m_state = Initialized;
}

ErrorBox::~ErrorBox()
{
    // remove from hash
    _instances->remove((quint64)m_parent);

    // in case applet is destroyed, main layout takes care about removing items
    if ( m_parent->layout() != 0 )
    {
        // remove from main layout
        for(int i = 0; i < m_parent->layout()->count(); ++i)
        {
            QGraphicsLayoutItem *item = m_parent->layout()->itemAt(i);

            if( item == this )
            {
                 m_parent->layout()->removeAt(i);

                break;
            }
        }
    }
}

void ErrorBox::exec(const QString &errMsg)
{
    // this function can be called multiple times during the life of object
    // everytime it should "restart" the object

    m_errorLabel->setText(errMsg);

    switch( m_state )
    {
        case Initialized:
        {
            m_fadeIn->start();
            m_state = FadingIn;

            break;
        }
        case FadingIn:
        {
            // just go on
            break;
        }
        case Running:
        {
            // restart lifetime

            m_lifetime.stop();
            m_lifetime.start();

            break;
        }
        case FadingOut:
        {
            // stop and restore
            m_fadeOut->stop();

            m_fadeIn->start();
            m_fadeIn->setCurrentTime( cFadeInDuration - m_fadeOut->currentTime() / 2 );

            m_state = FadingIn;

            break;
        }
        case Dead:
        {
            // go to Initialized
            m_fadeIn->start();

            m_state = FadingIn;

            break;
        }
        default:
        {
            // this should never happen
        }
    }
}

void ErrorBox::kill()
{
    switch ( m_state )
    {
        case Initialized:
        {
            // just go on

            break;
        }
        case FadingIn:
        {
            m_fadeIn->stop();

            break;
        }
        case Running:
        {
            m_lifetime.stop();

            break;
        }
        case FadingOut:
        {
           m_fadeOut->stop();

            break;
        }
        case Dead:
        {
           // just go on

            break;
        }
        default:
        {
            // this should never happen
        }
    }

    m_state = Dead;

    deleteLater();
}

void ErrorBox::fadeInFinished()
{
    m_lifetime.start();

    m_state = Running;
}

void ErrorBox::lifeTimeout()
{
    m_fadeOut->start();

    m_state = FadingOut;
}

void ErrorBox::fadeOutFinished()
{
    m_state = Dead;

    deleteLater();
}

#include "ErrorBox.moc"
