/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include <iostream>
// Qt
#include <QPainter>
#include <QByteArray>
#include <QFile>
#include <QSize>
#include <QRect>
#include <QColor>
#include <QGraphicsAnchorLayout>
#include <QPropertyAnimation>
#include <QParallelAnimationGroup>
#include <QEasingCurve>
#include <QTimer>

// KDE
#include <KDebug>
#include <KStandardDirs>
#include <KIcon>
#include <KConfigDialog>
#include <KServiceTypeTrader>
#include <KColorScheme>

// Plasma
#include <Plasma/ToolTipManager>
#include <Plasma/PaintUtils>
#include <Plasma/DataEngineManager>
#include <Plasma/DataContainer>
#include <Plasma/FrameSvg>

#include <math.h>

#include "potd.h"
#include "configdialog.h"
#include "ErrorBox.h"
#include "ActionsBox.h"

// shared constants
#include "potd_shared.h"

// 1 hour
#define cDefaultUpdateInterval   ((uint)3600000)
#define cDefaultSlideInterval    ((uint)600000)
#define cMinsInMs                ((uint)60000)
#define cHourInMs                ((uint)3600000)
#define c24HoursInMs             ((uint)86400000)
// 5 seconds
#define cErrorTimer              7500


PhotoOfTheDay::PhotoOfTheDay(QObject *parent, const QVariantList &args)
        : Plasma::Applet(parent, args)
{
    m_bAnimating = false;

    m_configDialog = 0;

    m_pEngine = NULL;

    m_pPhoto = NULL;
    m_pOldPhoto = NULL;

    m_slideTimer = NULL;

    m_slideShowEnabled = true;
    m_slideInterval = 60000; // 1 min

    setHasConfigurationInterface(true);

    setBackgroundHints(NoBackground);
    setCacheMode(QGraphicsItem::ItemCoordinateCache);

    // set main layout

    QGraphicsAnchorLayout *layout = new QGraphicsAnchorLayout(this);
    setLayout(layout);

    resize(400, 300);
}

PhotoOfTheDay::~PhotoOfTheDay()
{
    if ( NULL != m_pPhoto )
    {
        delete m_pPhoto;
    }

    if ( NULL != m_pOldPhoto )
    {
        delete m_pOldPhoto;
    }
}

void PhotoOfTheDay::init()
{
    // load configuration
    KConfigGroup cg = config();

    // General tab
    m_animatedTransition = cg.readEntry( "animatedTransition", true );

    m_updateInterval = cg.readEntry( "updateInterval", cDefaultUpdateInterval );

    if( ( m_updateInterval < cHourInMs ) || ( m_updateInterval > c24HoursInMs ) )
    {
        m_updateInterval = cDefaultUpdateInterval;
    }
    else
    {
        // round updateInterval to minutes (if some one edited config file manually)
        m_updateInterval = m_updateInterval - ( m_updateInterval % cMinsInMs );
    }

    m_slideShowEnabled = cg.readEntry( "slideShowEnabled", false );

    m_slideInterval = cg.readEntry( "slideInterval", cDefaultSlideInterval );

    if ( ( m_slideInterval < cMinsInMs ) || ( m_slideInterval > cHourInMs ) )
    {
        m_slideInterval = cDefaultSlideInterval;
    }
    else
    {
        m_slideInterval = m_slideInterval - ( m_slideInterval % cMinsInMs );
    }

    // Appearance tab
    m_hasFrame = cg.readEntry( "hasFrame", false );
    m_hasShadow = cg.readEntry( "hasShadow", true );
    m_hasRoundCorners = cg.readEntry( "roundCorners", false );
    m_smoothScaling = cg.readEntry( "smoothScaling", true );

    m_frameWidthConfig = cg.readEntry( "frameWidth", 5 );
    m_frameWidth = m_frameWidthConfig;
    m_shadowSizeConfig = cg.readEntry( "shadowSize", 5 );
    m_shadowSize = m_shadowSizeConfig;
    m_roundingFactor = cg.readEntry( "roundingFactor", 10 );

    m_frameOpacity = cg.readEntry("frameOpacity", 50);
    m_shadowOpacity = cg.readEntry("shadowOpacity", 50);

    m_frameColor = cg.readEntry("frameColor", QColor(220, 200, 20));
    m_shadowColor = cg.readEntry("shadowColor", QColor(10, 10, 10));

    // Actions box tab
    m_bShowActions = cg.readEntry( "showActionsBox", true );
    m_actionsBoxPosition = (ActionsBox::ActionBoxPosition)cg.readEntry( "actionsBoxPosition", (int)ActionsBox::BottomRight );

    if ( ActionsBox::PositionCount <= m_actionsBoxPosition )
    {
        m_actionsBoxPosition = ActionsBox::BottomRight;
    }

    loadDefaultImage();
    redrawApplet(contentSizeHint());
    update();

    // engines
    QString enginePluginName = cg.readEntry("engineName", QString("Default"));

    // query for all Photo of the day data engines
    KService::List offers = KServiceTypeTrader::self()->query("Plasma/DataEngine",
                                                              "[X-KDE-PluginInfo-Category] == 'PotdProvider'");

    if( !offers.isEmpty() )
    {
        foreach(const KService::Ptr &service, offers)
        {
            QString name = service->property("X-KDE-PluginInfo-Name").toString();

            if( !name.isEmpty() )
            {
                m_engineSources.append( name );
            }
        }
    }
    else
    {
        setErrorMessage( i18n( "There is no suitable data engines" ) );

        return;
    }

    if( !connectEngine(enginePluginName) )
    {
        setErrorMessage( QString( i18n( "Unable to load %1. Please select another engine" ) ).arg( enginePluginName ) );

        return;
    }

    startSlideShow();
}

void PhotoOfTheDay::createConfigurationInterface(KConfigDialog *parent)
{
    m_configDialog = new ConfigDialog(parent);

    parent->addPage(m_configDialog->potdSettings, i18n("General"), "games-config-background");
    parent->addPage(m_configDialog->appSettings, i18n("Appearance"), "preferences-desktop-theme");
    parent->addPage(m_configDialog->actionsBox, i18n("Actions box"), "preferences-desktop");

    parent->setDefaultButton(KDialog::Ok);
    parent->showButtonSeparator(true);

    // set all values
    // Appearance tab
    m_configDialog->setHasFrame(m_hasFrame);
    m_configDialog->setFrameWidth(m_frameWidthConfig);
    m_configDialog->setFrameOpacity(m_frameOpacity);
    m_configDialog->setFrameColor(m_frameColor);

    m_configDialog->setHasShadow(m_hasShadow);
    m_configDialog->setShadowSize(m_shadowSizeConfig);
    m_configDialog->setShadowOpacity(m_shadowOpacity);
    m_configDialog->setShadowColor(m_shadowColor);

    m_configDialog->setRoundedCorners(m_hasRoundCorners);
    m_configDialog->setRoundingFactor(m_roundingFactor);

    m_configDialog->setSmoothScaling(m_smoothScaling);

    // General tab
    m_configDialog->setAnamatedTransition(m_animatedTransition);
    m_configDialog->setUpdateInterval(m_updateInterval);

    m_configDialog->setSlideShowEnabled( m_slideShowEnabled );
    m_configDialog->setSlideInterval( m_slideInterval );

    // Actions box tab
    m_configDialog->setShowActions( m_bShowActions );
    m_configDialog->setActionBoxPosition( (int)m_actionsBoxPosition );

    // list data engines
    QStringList dataEnginesList;

    if( !m_engineSources.isEmpty() ) // if there are engines
    {
        foreach(const QString &de, m_engineSources)
        {
            Plasma::DataEngine *d = dataEngine(de);

            if(d->isValid())
            {
                dataEnginesList << d->name();
            }
        }

        m_configDialog->setDataEngines(dataEnginesList, m_pEngine->name());

    }
    else
    {
        //qDebug() << "PhotoOfTheDay::createConfigurationInterface() No engines avaible";
        // TODO: Show error message in dialog.
    }

    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

void PhotoOfTheDay::configAccepted()
{
    bool needReconnect;

    KConfigGroup cg = config();

    // saves new values
    // Appearance tab
    m_hasFrame = m_configDialog->getHasFrame();
    cg.writeEntry("hasFrame", m_hasFrame);
    m_hasShadow = m_configDialog->getHasShadow();
    cg.writeEntry("hasShadow", m_hasShadow);
    m_hasRoundCorners = m_configDialog->getRoundedCorners();
    cg.writeEntry("roundCorners", m_hasRoundCorners);
    m_smoothScaling = m_configDialog->getSmoothScaling();
    cg.writeEntry("smoothScaling", m_smoothScaling);

    m_frameWidthConfig = m_configDialog->getFrameWidth();
    cg.writeEntry("frameWidth", m_frameWidthConfig);
    m_frameWidth = m_frameWidthConfig;

    m_shadowSizeConfig = m_configDialog->getShadowSize();
    cg.writeEntry("shadowSize", m_shadowSizeConfig);
    m_shadowSize = m_shadowSizeConfig;

    m_roundingFactor = m_configDialog->getRoundingFactor();
    cg.writeEntry("roundingFactor", m_roundingFactor);

    m_frameOpacity = m_configDialog->getFrameOpacity();
    cg.writeEntry("frameOpacity", m_frameOpacity);
    m_shadowOpacity = m_configDialog->getShadowOpacity();
    cg.writeEntry("shadowOpacity", m_shadowOpacity);

    m_frameColor = m_configDialog->getFrameColor();
    cg.writeEntry("frameColor", m_frameColor);

    m_shadowColor = m_configDialog->getShadowColor();
    cg.writeEntry("shadowColor", m_shadowColor);

    // General tab
    m_animatedTransition = m_configDialog->getAnamatedTransition();
    cg.writeEntry("animatedTransition", m_animatedTransition);

    uint updateInterval = m_configDialog->getUpdateInterval();
    cg.writeEntry("updateInterval", updateInterval);

    bool slideShowEnabled = m_configDialog->getSlideShowEnabled();
    cg.writeEntry( "slideShowEnabled", slideShowEnabled );

    uint slideShowInterval = m_configDialog->getSlideInterval();
    cg.writeEntry( "slideInterval" , slideShowInterval );

    // Actions box tab
    m_bShowActions = m_configDialog->getShowActions();
    cg.writeEntry( "showActionsBox", m_bShowActions );
    
    m_actionsBoxPosition = (ActionsBox::ActionBoxPosition)m_configDialog->getActionBoxPosition();
    cg.writeEntry("actionsBoxPosition", (int)m_actionsBoxPosition);


    QString newEngineName = m_configDialog->getDataEngine();

    if( updateInterval != m_updateInterval )
    {
        needReconnect = true;
        m_updateInterval = updateInterval;
    }
    else
    {
        needReconnect = false;
    }

    if( newEngineName != m_pEngine->name() )
    {

        // query for all Photo of the day data engines
        KService::List offers = KServiceTypeTrader::self()->query("Plasma/DataEngine",
                                                                  QString("Name == '%1' && [X-KDE-PluginInfo-Category] == 'PotdProvider' ").arg(newEngineName));

        //qDebug() << "Offers are " << offers.count();

        if( ( !offers.isEmpty() ) && ( offers.count() == 1 ) )
        {
            const KService::Ptr & dePtr = offers.first();

            QString pluginName = dePtr->property("X-KDE-PluginInfo-Name").toString();

            if(!pluginName.isEmpty())
            {
                // new engine is ok
                //disconnect old engine
                disconnectEngine();

                if( connectEngine(pluginName) )
                {
                    // only if everything is ok save new engine
                    cg.writeEntry("engineName", pluginName);

                    // new engine has been loaded and new update interval is applied
                    needReconnect = false;
                }
                else
                {
                    setErrorMessage(i18n("Could not connect to new engine."));
                }
            }
            else
            {
                setErrorMessage(i18n("X-KDE-PluginInfo-Name is empty."));
            }
        }
        else
        {
            setErrorMessage(i18n("There is no data engine with that name."));
        }
    }

    // we are connected to engine and we need to reconnect to apply new update interval
    if( ( NULL != m_pEngine ) && ( true == needReconnect ) )
    {
        QString pluginName = m_pEngine->pluginName();
        disconnectEngine();
        // we have been connected to this engine so we know it can be connected again
        (void)connectEngine( pluginName );
    }

    // slide show timer
    if ( slideShowInterval != m_slideInterval )
    {
        if ( NULL != m_slideTimer )
        {
            m_slideInterval = slideShowInterval;
            m_slideTimer->stop();
            m_slideTimer->setInterval( m_slideInterval );
            m_slideTimer->start();
        }
    }


    if ( slideShowEnabled )
    {
        if ( m_slideShowEnabled )
        {
            // on -> on ( we have running slide show timer );

            if ( slideShowInterval != m_slideInterval )
            {
                // timer need restart
                m_slideInterval = slideShowInterval;

                m_slideTimer->stop();
                m_slideTimer->setInterval( m_slideInterval );
                m_slideTimer->start();
            }
        }
        else
        {
            // off -> on ( start slide show timer now )

            m_slideShowEnabled = slideShowEnabled;
            m_slideInterval = slideShowInterval;

            startSlideShow();
        }
    }
    else
    {
        m_slideInterval = slideShowInterval;

        if ( m_slideShowEnabled )
        {
            // on -> off ( remove slide show timer )

            m_slideShowEnabled = slideShowEnabled;

            disconnect( m_slideTimer, SIGNAL( timeout() ), this, SLOT( useNextEngine() ) );
            m_slideTimer->stop();
            delete m_slideTimer;

            m_slideTimer = NULL;
        }
        // else off -> off => do nothing"
    }

    redrawApplet(contentSizeHint());
    update();

    emit configNeedsSaving();
}


/*
    Sets tooltip with current photo metainfo
*/
void PhotoOfTheDay::updateToolTip()
{
    Plasma::ToolTipContent data(m_title, m_info, KIcon("dialog-information"));
    data.setAutohide(false);
    Plasma::ToolTipManager::self()->setContent(this, data);
}


void PhotoOfTheDay::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint)
    {
        setBackgroundHints(Plasma::Applet::NoBackground);

        if (formFactor() == Plasma::Horizontal)
        {
            m_frameWidth = 0;
            m_shadowSize = m_shadowSizeConfig / 2;
        } else if (formFactor() == Plasma::Vertical)
        {
            m_frameWidth = 0;
            m_shadowSize = m_shadowSizeConfig / 2;
        } else
        {
            m_frameWidth = m_frameWidthConfig;
            m_shadowSize = m_shadowSizeConfig;
            //Restore widget geometry to image proportions
           // if( false == m_bAnimating )
            //{
                resize(contentSizeHint());
                emit appletTransformedItself();
           // }
           // else
           // {
                // TODO: Do I need to do something here ???
           // }
        }

       // updatePhoto();
    }

    if (constraints & Plasma::SizeConstraint)
    {
        //If on panel, keep geometry to 4:3 ratio
        if(formFactor() == Plasma::Vertical)
        {
            setMinimumSize(QSizeF(0, contentsRect().width()/1.33));
            setMaximumSize(QSizeF(-1, contentsRect().width()/1.33));
        } else if(formFactor() == Plasma::Horizontal)
        {
            setMinimumSize(QSizeF(contentsRect().height()*1.33,0));
            setMaximumSize(QSizeF(contentsRect().height()*1.33,-1));
        } else
        {
            setMinimumSize(QSizeF());
            setMaximumSize(QSizeF());
        }
    }
}

//======================================================================================================================
// Engine related functions
//======================================================================================================================

// This function updates new data from currently selected engine
void PhotoOfTheDay::dataUpdated(const QString& sourceName, const Plasma::DataEngine::Data& data)
{
    if( I18N_NOOP(cPotdPhotoSource) == sourceName )
    {
        QPixmap photo = data["Photo"].value<QPixmap>();

        if( isBusy() )
        {
            setBusy(false);
        }

        if( !photo.isNull() )
        {
            if( true == m_animatedTransition)
            {
                if ( true == m_bAnimating )
                {
                    // stop animation
                    stopTranstionAnimation();
                }

                m_pOldPhoto = m_pPhoto;
                m_pPhoto = new QPixmap( photo );

                startTransitionAnimation();
            }
            else
            {
                // just set new photo and update

                delete m_pPhoto;

                m_pPhoto = new QPixmap( photo );

                //redrawApplet(contentSizeHint());
                resize( contentSizeHint() );
                update();
            }

            m_title = data["Title"].toString();
            m_info = data["Info"].toString();

            setAssociatedApplicationUrls( data["Url"].value<KUrl>() );

            updateToolTip();
        }

    }
    else if(I18N_NOOP(cPotdErrorSource) == sourceName)
    {
        const QString error = data["Error"].toString();

        if(!error.isEmpty())
        {
            setErrorMessage(error);
            update();
        }
    }
    else
    {
        Q_ASSERT_X(false, "PhotoOfTheDay::dataUpdated", "Invalid source!");
    }
}

bool PhotoOfTheDay::connectEngine(const QString & pluginName)
{
    bool bRet = true;

    if( !pluginName.isEmpty() )
    {
        //qDebug()<< "Loading engine: " << pluginName;

        if( "Default" == pluginName )
        {
            m_pEngine = dataEngine(m_engineSources.first());
        }
        else
        {
            m_pEngine = dataEngine(pluginName);
        }

        if( m_pEngine->isValid() )
        {
             // set up connections for updates
            m_pEngine->connectSource(cPotdPhotoSource, this, m_updateInterval);
            m_pEngine->connectSource(cPotdErrorSource, this);

            // when we connect to data engine for first time data will be empty, so wait for dataUpdated event
            // else just call dataUpdated
            Plasma::DataEngine::Data data = m_pEngine->query( cPotdPhotoSource );
            if ( data.isEmpty() )
            {
                if( !isBusy() )
                {
                    setBusy(true);
                }
            }
            else
            {
                dataUpdated( cPotdPhotoSource, data );
            }
        }
        else
        {
            bRet = false;
        }
    }
    else
    {
        bRet = false;
    }


    return bRet;
}

void PhotoOfTheDay::disconnectEngine()
{
    m_pEngine->disconnectSource(cPotdPhotoSource, this);
    m_pEngine->disconnectSource(cPotdErrorSource, this);

    m_pEngine = NULL;
}

void PhotoOfTheDay::useNextEngine()
{
    QListIterator<QString> iter( m_engineSources );

    iter.findNext( m_pEngine->pluginName() );

    if( ! iter.hasNext() )
    {
         iter.toFront();
    }

    disconnectEngine();
    connectEngine( iter.next() );
}

void PhotoOfTheDay::usePrevEngine()
{
    QListIterator<QString> iter( m_engineSources );

    iter.toBack();

    iter.findPrevious( m_pEngine->pluginName() );

    if( ! iter.hasPrevious() )
    {
        iter.toBack();
    }

    disconnectEngine();
    connectEngine( iter.previous() );
}

//======================================================================================================================
// Drawing and painting
//======================================================================================================================

void PhotoOfTheDay::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect& rect)
{
    Q_UNUSED(option);

    if( m_drawnApplet.isNull() )
    {
        qDebug() << "drawing NULL m_photo !!!";
        return; // this should never happen
    }

    redrawApplet(rect.size());

    painter->drawPixmap(rect, m_drawnApplet, rect);
}

void PhotoOfTheDay::redrawApplet(const QSizeF &newSize)
{
    QPixmap drawable( newSize.toSize() );
    drawable.fill( Qt::transparent );

    QPainter *p = new QPainter();
    p->begin( &drawable );

    // calculate roundness of frame and shadow if any
    int roundingFactor = m_roundingFactor * (int)m_hasRoundCorners; // false = 0
    int shadowRoundness = roundingFactor + ( m_frameWidth * 2 ) * (int)m_hasFrame * (int)m_hasRoundCorners;


    QRectF drawingRect( QPointF( 0, 0 ), newSize );

    p->setRenderHint( QPainter::SmoothPixmapTransform, true );
    p->setRenderHint( QPainter::Antialiasing, true );

    // if we have a shadow draw it and resize rectangle so in the end we will have remaining rect for photo
    if ( m_hasShadow )
    {
        p->setBrush( Qt::NoBrush );

        QPen pen( Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin );

        QEasingCurve shadowCurve( QEasingCurve::InCubic );

        // draw m_shadowSize lines with increasing opacity
        for ( int i = 1; i <= m_shadowSize + 1 ; ++i )
        {
            //qreal opacity = (qreal)((qreal)(i) / (qreal)( m_shadowSize + 1 ) ) * ((qreal)m_shadowOpacity / 100.0);
            qreal opacity = shadowCurve.valueForProgress( (qreal)(i) / (qreal)( m_shadowSize + 1 ) ) * ( (qreal)m_shadowOpacity / 100.0 );
            m_shadowColor.setAlphaF(opacity);
            pen.setColor(m_shadowColor);
            p->setPen(pen);

            QPainterPath shadowPath = Plasma::PaintUtils::roundedRectangle( drawingRect, shadowRoundness + ( m_shadowSize - i ) );
            p->drawPath(shadowPath);

            drawingRect.adjust( +1.0, +1.0, -1.0, -1.0 );
        }

        drawingRect.adjust( -1.0, -1.0, 1.0, 1.0 );
    }

    p->setBrush( Qt::NoBrush );


    // if we have a frame draw it
    QPainterPath framePath = Plasma::PaintUtils::roundedRectangle(drawingRect, roundingFactor);


    if ( m_hasFrame )
    {
        // adjust rectangle to inner side of frame
        drawingRect.adjust(+(qreal)m_frameWidth, +(qreal)m_frameWidth, -(qreal)m_frameWidth, -(qreal)m_frameWidth);

        m_frameColor.setAlphaF((qreal)m_frameOpacity/100.0);

        // The frame is painted twice as thick, the inner half lies behind the picture
        // This is important to not make the corners look "cut out" when rounded
        p->setPen(QPen(m_frameColor, (qreal)(m_frameWidth * 2), Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
        framePath = Plasma::PaintUtils::roundedRectangle(drawingRect, roundingFactor);

        p->drawPath(framePath);
    }

    // We save the painter here so the clipping only happens on the pixmap,
    // not on pixmap, frame, shadow, etc
    p->save();

    if (m_hasRoundCorners)
    {
        p->setClipPath(framePath);
    }

    // Respect the smooth-scaling setting
    p->setRenderHint(QPainter::SmoothPixmapTransform, m_smoothScaling);
    // draw our pixmap into the computed rectangle
    Qt::TransformationMode trMode = m_smoothScaling ? Qt::SmoothTransformation : Qt::FastTransformation;

    // get photo
    QPixmap pm;
    if ( false == m_bAnimating )
    {
        pm = m_pPhoto->scaled( drawingRect.size().toSize(), Qt::KeepAspectRatio, trMode );
    }
    else
    {
        QPixmap from = m_pOldPhoto->scaled( drawingRect.size().toSize(), Qt::IgnoreAspectRatio, Qt::FastTransformation );
        QPixmap to = m_pPhoto->scaled( drawingRect.size().toSize(), Qt::IgnoreAspectRatio, Qt::FastTransformation );

        pm = Plasma::PaintUtils::transition( from, to, m_transtionOpacity );
    }

    p->drawPixmap(drawingRect.toRect(), pm);
    p->restore();

    p->setRenderHint(QPainter::SmoothPixmapTransform, true);

    // black frame
    if ( m_hasFrame || m_hasRoundCorners )
    {
        p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
        p->drawPath(framePath);
    }

    p->end();
    delete p;

    m_drawnApplet = drawable;
}

void PhotoOfTheDay::loadDefaultImage()
{
    Plasma::FrameSvg * background = new Plasma::FrameSvg( this );

    background->setImagePath( "widgets/background" );
    background->setEnabledBorders( Plasma::FrameSvg::NoBorder );
    background->resizeFrame( boundingRect().size() );

    m_pPhoto = new QPixmap( background->framePixmap() );
}

//======================================================================================================================
// Calculations
//======================================================================================================================

QSizeF PhotoOfTheDay::contentSizeHint() const
{
    // we use pixmap to calculate new size because total offset does not effect ratio
    QSizeF size = m_pPhoto->size();

    if( ( !size.isEmpty() ) && ( ( formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter ) ) )
    {
        const qreal maxSize = qMax(contentsRect().width(), contentsRect().height());

        size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
        return size;
    }
    else
    {
        return contentsRect().size();
    }
}

QSizeF PhotoOfTheDay::transitionSizeHint(const QPixmap &pm) const
{
    QSizeF size = pm.size();

    if(!size.isEmpty() &&
       (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter))
    {
        const qreal maxSize = qMax(contentsRect().width(), contentsRect().height());

        size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
        return size;
    }
    else
    {
        return contentsRect().size();
    }
}

//======================================================================================================================
// Error handling
//======================================================================================================================

void PhotoOfTheDay::setErrorMessage(const QString &errP)
{
    ErrorBox::Instance(this)->exec(errP);
}

//======================================================================================================================
// Mouse hover events handling
//======================================================================================================================

void PhotoOfTheDay::hoverEnterEvent( QGraphicsSceneHoverEvent *event )
{
    Q_UNUSED( event );

    if ( m_bShowActions )
    {
        ActionsBox::Instance( m_actionsBoxPosition, this )->exec();
    }
}

void PhotoOfTheDay::hoverLeaveEvent( QGraphicsSceneHoverEvent *event )
{
    Q_UNUSED( event );

    if ( m_bShowActions )
    {
        ActionsBox::Instance( m_actionsBoxPosition, this)->hideActions();
    }
}

//======================================================================================================================
// Transition animation
//======================================================================================================================

void PhotoOfTheDay::startTransitionAnimation()
{
    m_bAnimating = true;

    m_pOpacityAnim = new QPropertyAnimation( this, "transtionOpacity", this );
    m_pOpacityAnim->setStartValue( 0.0 );
    m_pOpacityAnim->setEndValue( 1.0 );
    m_pOpacityAnim->setDuration( 2000 );

    m_pSizeAnim = new QPropertyAnimation( this, "size", this );
    m_pSizeAnim->setStartValue( contentsRect().size() );
    m_pSizeAnim->setEndValue( transitionSizeHint( *m_pPhoto ) );
    m_pSizeAnim->setDuration( 2000 );

    m_pAnimGroup = new QParallelAnimationGroup( this );
    m_pAnimGroup->addAnimation( m_pOpacityAnim );
    m_pAnimGroup->addAnimation( m_pSizeAnim );

    connect( m_pAnimGroup, SIGNAL(finished()), this, SLOT(animationFinished()) );

    m_pAnimGroup->start();
}

void PhotoOfTheDay::stopTranstionAnimation()
{
    m_pAnimGroup->stop();

    delete m_pOpacityAnim;
    delete m_pSizeAnim;
    delete m_pAnimGroup;

    QPixmap from = m_pOldPhoto->scaled( boundingRect().size().toSize(), Qt::IgnoreAspectRatio, Qt::FastTransformation );
    QPixmap to = m_pPhoto->scaled( boundingRect().size().toSize(), Qt::IgnoreAspectRatio, Qt::FastTransformation );

    delete m_pOldPhoto;

    *m_pPhoto = Plasma::PaintUtils::transition( from, to, m_transtionOpacity );

    m_bAnimating = false;
}

void PhotoOfTheDay::animationFinished()
{
    m_bAnimating = false;

    update();
}

qreal PhotoOfTheDay::getTranstionOpacity() const
{
    return m_transtionOpacity;
}

void PhotoOfTheDay::setTranstionOpacity(qreal opacity)
{
    m_transtionOpacity = opacity;

    update();
}

//======================================================================================================================
// Inline functions
//======================================================================================================================
int PhotoOfTheDay::calcOffset()
{
    int offset = m_shadowSize * (int)m_hasShadow;

    offset += ( m_frameWidth * (int)m_hasFrame );

    offset += (int)( m_hasFrame || m_hasShadow );

    return offset;
}

void PhotoOfTheDay::startSlideShow()
{
    // function will start slide show only if it configured.

    // start slide show if there is more than one engine and timer is not started
    if ( m_slideShowEnabled && ( m_engineSources.size() > 1 ) && ( NULL == m_slideTimer ) )
    {
        m_slideTimer = new QTimer( this );
        m_slideTimer->setInterval( m_slideInterval );
        m_slideTimer->start();

        connect( m_slideTimer, SIGNAL( timeout() ), this, SLOT( useNextEngine() ) );
    }
}



K_EXPORT_PLASMA_APPLET(photo_of_the_day, PhotoOfTheDay)

#include "potd.moc"
