 /***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "AbstractProvider.h"

// Qt
#include <QFile>
#include <QSettings>

// KDE
#include <KStandardDirs>

AbstractProvider::AbstractProvider(QObject *parent, const QVariantList &args):
        Plasma::DataEngine(parent, args)
{

    connect(Solid::Networking::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)), this, SLOT(networkStatusChangedSlot(Solid::Networking::Status)));
}

void AbstractProvider::networkStatusChangedSlot(Solid::Networking::Status status)
{
    // if we have an internet update photo
    if( (Solid::Networking::Connected == status) || (Solid::Networking::Unknown) )
    {
        // TODO: set protection if source doesn`t exist yet
        sourceRequestEvent(cPotdPhotoSource);

        forceImmediateUpdateOfAllVisualizations();
    }
}

void AbstractProvider::setPhotoSourceData()
{
    setData( cPotdPhotoSource, "Photo", m_photo );
    setData( cPotdPhotoSource, "Title", m_title );
    setData( cPotdPhotoSource, "Info", m_info );
    setData( cPotdPhotoSource, "Url", m_associatedUrl );
}

void AbstractProvider::setError( const QString &err )
{
    setData( cPotdErrorSource, "Error", err );
}

void AbstractProvider::restore()
{
    if ( m_photo.isNull() )
    {
        // no old data. First request failed. Load from HDD
        if ( !loadCache() )
        {
            // failed to load from HDD.
            setError( "Failed to load from local cache." );
        }
    }
    else
    {
        // restore from engine old values
        Data d = query( cPotdPhotoSource );

        m_photo = d.value("Photo").value<QPixmap>();
        m_title = d.value("Title").value<QString>();
        m_info = d.value("Info").value<QString>();
        m_associatedUrl = d.value("Url").value<KUrl>();
    }

    setPhotoSourceData();
}

void AbstractProvider::saveCache() const
{
    saveText();
    savePixmap();
}

void AbstractProvider::saveText() const
{
    QString dir = KStandardDirs::locateLocal( "data", "photo_of_the_day/" );

    QSettings photoMetaInfo( dir + "PhotoMetaInfo.conf", QSettings::IniFormat );
    photoMetaInfo.beginGroup( pluginName() );
    photoMetaInfo.setValue( "title", m_title );
    photoMetaInfo.setValue( "info", m_info );
    photoMetaInfo.setValue( "url", m_associatedUrl.prettyUrl() );
    photoMetaInfo.endGroup();

    photoMetaInfo.sync();
}

void AbstractProvider::savePixmap() const
{
    if ( m_photo.isNull() ) return;

    QString dir = KStandardDirs::locateLocal( "data", "photo_of_the_day/" );
    QString cachePath = dir + pluginName() + "_cache";

    QFile cachedPhoto(cachePath);

    //qDebug() << "Saving cache to " << cachePath;

    if ( cachedPhoto.exists() )
    {
        (void)cachedPhoto.remove();
    }

    if( ! m_photo.save(cachePath, "JPG") )
    {
        qDebug() << "Saving failed!";

        return;
    }
}

bool AbstractProvider::loadCache()
{
    bool result = true;

    loadText();

    result = loadPixmap();

    if ( !result )
    {
        loadText();
    }

    return result;
}

void AbstractProvider::loadText()
{
    QString dir = KStandardDirs::locateLocal( "data", "photo_of_the_day/" );

    QSettings photoMetaInfo( dir + "PhotoMetaInfo.conf", QSettings::IniFormat );
    photoMetaInfo.beginGroup( pluginName() );
    m_title = photoMetaInfo.value( "title", QString() ).toString();
    m_info = photoMetaInfo.value( "info", QString() ).toString();
    m_associatedUrl = KUrl( photoMetaInfo.value( "url", QString() ).toString() );
    photoMetaInfo.endGroup();
}

bool AbstractProvider::loadPixmap()
{
    bool result = true;

    QString dir = KStandardDirs::locateLocal("data", "photo_of_the_day/");
    QString cachePath = dir + pluginName() + "_cache";

    if ( QFile::exists(cachePath) )
    {
        if( !m_photo.load( cachePath ) )
        {
            result = false;
        }
    }
    else
    {
        result = false;
    }

    return result;
}

