 /***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// KDE
#include <KIO/Job>

#include "Apotd.h"


const char *cMainPageUrl = "http://antwrp.gsfc.nasa.gov/apod/astropix.html";
const char *cPhotosWebsite = "http://antwrp.gsfc.nasa.gov/apod/image/";

Apotd::Apotd(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
    m_associatedUrl = KUrl( cMainPageUrl );
}

Apotd::~Apotd()
{
}


// called on first source request
bool Apotd::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if(I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X(false, "Apotd::sourceRequestEvent()", "Invalid source!");

        return result = false;
    }


    return result;
}

bool Apotd::updateSourceEvent (const QString &source)
{
    bool result = true;

    if( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet(KUrl(cMainPageUrl), KIO::Reload, KIO::HideProgressInfo);

        connect(job, SIGNAL(result(KJob*)), this, SLOT(mainPageLoaded(KJob*)));
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        // do nothing for now
    }
    else
    {
        //Q_ASSERT_X(false, "Apotd::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void Apotd::mainPageLoaded( KJob *job )
{
    if( job->error() )
    {
        setError( "[FATAL ERROR]Apotd Engine: Unable to connect.\nLoading from cache ..." );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        parseMainPage(storedJob->data());
    }
}

void Apotd::parseMainPage( QByteArray source )
{
    int photoStart = source.indexOf("<IMG SRC=\"image/") + (int)strlen("<IMG SRC=\"image/");
    int photoEnd = source.indexOf("\"", photoStart);

    KUrl newUrl = KUrl(cPhotosWebsite + source.mid(photoStart, photoEnd - photoStart));

    if(!newUrl.isValid())
    {
        setError( QString("[ERROR]Invalid url: %1.").arg( newUrl.url() ) );
        // TODO: add 'loading from cache ...' only if m_photo.isNull

        restore();

        return;
    }

    if(newUrl != m_OldUrl)
    {
        // we have new photo
        m_OldUrl = newUrl;

        int titleStart = source.indexOf("<b>", photoEnd) + 3;
        int titleEnd = source.indexOf("</b>", titleStart);

        m_title = source.mid(titleStart, titleEnd - titleStart).trimmed();

        // remove a lot of byte array that would be used
        QString pageSource(source.remove(0, titleEnd));

        // remove html tags
        pageSource.replace(QRegExp("<[^>]*>"), "");

        int infoStart = pageSource.indexOf("Explanation:") + strlen("Explanation:");
        int infoEnd = pageSource.indexOf("digg_url", infoStart);

        m_info = pageSource.mid(infoStart, infoEnd - infoStart);

        KIO::StoredTransferJob *job = KIO::storedGet( m_OldUrl, KIO::Reload, KIO::HideProgressInfo );
        connect(job, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        // photo is the same as old one
        setPhotoSourceData();
        forceImmediateUpdateOfAllVisualizations();
    }
}

void Apotd::photoDownloaded(KJob *job)
{
    if(job->error())
    {
        setError( QString("[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ...").arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);


        if( m_photo.loadFromData( storedJob->data() ) )
        {
            // everything is ok
            setPhotoSourceData();
            scheduleSourcesUpdated();

            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load pohto. Loading from cashe ...");

            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(ApotdPotd, Apotd)

#include "Apotd.moc"

