/***************************************************************************
 *   Copyright 2009-2010 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/ 

// KDE
#include <KIO/Job>

#include "EarthShots.h"

const char *cMainPageUrl = "http://www.earthshots.org/";

EarthShots::EarthShots(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
}

EarthShots::~EarthShots()
{
}


bool EarthShots::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if (I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if (I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X( false, "EarthShots::sourceRequestEvent()", "Invalid source!" );

        return result = false;
    }


    return result;
}

bool EarthShots::updateSourceEvent (const QString &source)
{
    bool result = true;

    if ( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet( KUrl( cMainPageUrl ), KIO::Reload, KIO::HideProgressInfo );

        connect( job, SIGNAL(result(KJob*)), this, SLOT(mainPageLoaded(KJob*)) );
    }
    else if ( I18N_NOOP( cPotdErrorSource ) == source )
    {
        // do nothing for now
    }
    else
    {
        //Q_ASSERT_X(false, "NatGeo::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void EarthShots::mainPageLoaded(KJob *job)
{
    if ( job->error() )
    {
        setError( "[FATAL ERROR]EarthShots Engine: Unable to connect.\nLoading from cache ..." );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);

        // parse main page for a link to todays photo details page

        QByteArray page = storedJob->data();

        int urlStart = page.indexOf("<div class=\"media\"><a href=\"");

        urlStart += QString("<div class=\"media\"><a href=\"").length();

        int urlEnd = page.indexOf("\">", urlStart);

        KUrl photoDetailsPage = KUrl( page.mid(urlStart, urlEnd - urlStart) );

        qDebug() << "Details page: " << photoDetailsPage;

        if ( photoDetailsPage != m_OldUrl ) // a new day has come :)
        {
            m_OldUrl = photoDetailsPage;

            m_associatedUrl = photoDetailsPage;

            // request details page

            KIO::StoredTransferJob *newJob = KIO::storedGet( photoDetailsPage, KIO::Reload, KIO::HideProgressInfo );

            connect(newJob, SIGNAL(result(KJob*)), this, SLOT(detailsPageLoaded(KJob*)));
        }
        else
        {
            setPhotoSourceData();
        }
    }
}

void EarthShots::detailsPageLoaded(KJob *job)
{
    if (job->error() == 0) // ok. parse it
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        QByteArray page = storedJob->data();

        // now parse details page
        int photoLinkStart = page.indexOf("<span id=\"img-container\"><img src=\"")
                               + QString("<span id=\"img-container\"><img src=\"").length();
        int photoLinkEnd =  page.indexOf("\"", photoLinkStart);
        QString photoLink = page.mid(photoLinkStart, photoLinkEnd - photoLinkStart);
        qDebug() << "EarthShots::detailsPageLoaded() photo link is: " << photoLink;

        // first we start with photo itself

        // than find title
        int titleStart = page.indexOf("<div class=\"article\">", photoLinkEnd);
        titleStart = page.indexOf("<h1>", titleStart) + 4;
        int titleEnd = page.indexOf("</h1>", titleStart); // first closed h1 tag after start

        m_title = page.mid(titleStart, titleEnd - titleStart);

        // and caption
        int captionStart = page.indexOf("<div id=\"es_win_caption\">", titleEnd) +
                           QString("<div id=\"es_win_caption\">").length();
        int captionEnd = page.indexOf("</div>", captionStart); // first closed div tag after start



        // and finally photographer
        int photogStart = page.indexOf("<div id=\"es_win_photog\">", captionEnd);

        if ( -1 != photogStart  )
        {
            photogStart += QString("<div id=\"es_win_photog\">").length();

            int photogEnd = page.indexOf("</div>", photogStart); // fist closed div tag after start

            m_info = "Photo by " + page.mid(photogStart, photogEnd - photogStart) +
                     "<br>" + page.mid(captionStart, captionEnd - captionStart);
        }
        else
        {
            m_info = page.mid(captionStart, captionEnd - captionStart);
        }

        // now download photo
        KIO::StoredTransferJob *newJob = KIO::storedGet(photoLink, KIO::Reload, KIO::HideProgressInfo);
        connect(newJob, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        setError( "Unable to load details page" );
        restore();
    }
}

void EarthShots::photoDownloaded(KJob *job)
{
    if (job->error())
    {
        setError( QString( "[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ..." ).arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);


        if ( m_photo.loadFromData(storedJob->data()) )
        {
            setPhotoSourceData();
            scheduleSourcesUpdated();
            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load pohto. Loading from cashe ..." );
            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(EarthShotsPotd, EarthShots);

#include "EarthShots.moc"
