 /***************************************************************************
 *   Copyright 2009-2010 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// KDE
#include <KIO/Job>

#include "NationalGeographic.h"


const char *cMainPageUrl = "http://photography.nationalgeographic.com/photography/photo-of-the-day";
const char *cPhotosWebsite = "http://photography.nationalgeographic.com";

NatGeo::NatGeo(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
    // all photos have the same page
    m_associatedUrl = KUrl( cMainPageUrl );
}

NatGeo::~NatGeo()
{
}


// called on first source request
bool NatGeo::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if(I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X(false, "NatGeo::sourceRequestEvent()", "Invalid source!");

        return result = false;
    }


    return result;
}

bool NatGeo::updateSourceEvent (const QString &source)
{
    bool result = true;

    if( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet(KUrl(cMainPageUrl), KIO::Reload, KIO::HideProgressInfo);

        connect(job, SIGNAL(result(KJob*)), this, SLOT(mainPageLoaded(KJob*)));

    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        // do nothing for now
    }
    else
    {
        //Q_ASSERT_X(false, "NatGeo::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void NatGeo::mainPageLoaded(KJob *job)
{
    if(job->error())
    {
        setError( "[FATAL ERROR]NatGeo Engine: Unable to connect.\nLoading from cache ..." );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        parseMainPage(storedJob->data());
    }
}

void NatGeo::parseMainPage(QByteArray source)
{
    int primaryPhotoDiv = source.indexOf("<div class=\"primary_photo\">");

    int imgSrcStart = source.indexOf( "<img src=\"", primaryPhotoDiv ) + QString( "<img src=\"" ).length();
    int imgSrcStop = source.indexOf( "\"", imgSrcStart );
    KUrl newUrl = KUrl(source.mid(imgSrcStart, imgSrcStop - imgSrcStart));

    if(!newUrl.isValid())
    {
        setError( QString( "[ERROR]Invalid url: %1." ).arg( newUrl.url() ) );

        restore();

        return;
    }

    if(newUrl != m_OldUrl)
    {
        // we have new photo
        m_OldUrl = newUrl;

        // get title
        int podsummaryStart = source.indexOf("<div id=\"caption\">", imgSrcStop);
        podsummaryStart = source.indexOf( "<h2>", podsummaryStart ) + 4;

        int podsummaryEnd = source.indexOf( "</h2>", podsummaryStart );
        m_title = source.mid( podsummaryStart, ( podsummaryEnd - podsummaryStart) );

        // get photograph
        int photographStart = source.indexOf("<p class=\"credit\">", podsummaryEnd);
        photographStart += QString("<p class=\"credit\">").length();
        int photographEnd = source.indexOf("</p>", photographStart);
        QString photograph = source.mid(photographStart, photographEnd - photographStart);

        // get description
        int descriptionStart = source.indexOf("</em></p>", photographEnd) + QString("</em></p>").length();
        descriptionStart = source.indexOf("<p>", descriptionStart) + 3;
        int descriptionEnd = source.indexOf("</p>", descriptionStart);
        m_info = photograph + "<br/>" + source.mid(descriptionStart, descriptionEnd - descriptionStart);

        // we have the photo URL so download it
        KIO::StoredTransferJob *job = KIO::storedGet(m_OldUrl, KIO::Reload, KIO::HideProgressInfo);
        connect(job, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        // photo is the same as old one
        setPhotoSourceData();
    }
}

void NatGeo::photoDownloaded(KJob *job)
{
    if(job->error())
    {
        setError( QString("[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ...").arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);


        if( m_photo.loadFromData(storedJob->data()) )
        {
            // everything is ok

            setPhotoSourceData();
            scheduleSourcesUpdated();

            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load pohto. Loading from cashe ..." );

            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(NatGeoPotd, NatGeo)

#include "NationalGeographic.moc"
