 /***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// KDE
#include <KIO/Job>

#include "StevesDigicams.h"


const char *cMainPageUrl = "http://ibpub.internetbrands.com/cgi-bin/mt/ib-potd-widget.cgi?render=full";
const char *cPhotosWebsite = "http://www.steves-digicams.com/photo-of-the-day/pub/";
const char *cPhotoTag = "<img src=\"http://www.steves-digicams.com/photo-of-the-day/pub/";

StevesDigicams::StevesDigicams(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
    m_associatedUrl = KUrl( cMainPageUrl );
}

StevesDigicams::~StevesDigicams()
{
}

// called on first source request
bool StevesDigicams::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if(I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X(false, "StevesDigicams::sourceRequestEvent()", "Invalid source!");

        return result = false;
    }


    return result;
}

bool StevesDigicams::updateSourceEvent (const QString &source)
{
    bool result = true;

    if( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet(KUrl(cMainPageUrl), KIO::Reload, KIO::HideProgressInfo);

        connect(job, SIGNAL(result(KJob*)), this, SLOT(mainPageLoaded(KJob*)));
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        // do nothing for now
    }
    else
    {
        //Q_ASSERT_X(false, "StevesDigicams::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void StevesDigicams::mainPageLoaded( KJob *job )
{
    if( job->error() )
    {
        setError( "[FATAL ERROR]StevesDigicams Engine: Unable to connect.\nLoading from cache ..." );
        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        parseMainPage( storedJob->data() );
    }
}

void StevesDigicams::parseMainPage(QByteArray source)
{
    int photoStart = source.indexOf(cPhotoTag) + (int)strlen(cPhotoTag);
    int photoEnd = source.indexOf("\">", photoStart);

    KUrl newUrl = KUrl( cPhotosWebsite + source.mid(photoStart, photoEnd - photoStart) );

    if(!newUrl.isValid())
    {
        setError( QString("[ERROR]Invalid url: %1.").arg( newUrl.url() ) );
        // TODO: add 'loading from cache ...' only if m_photo.isNull

        restore();

        return;
    }

    if(newUrl != m_OldUrl)
    {
        // we have new photo

        m_OldUrl = newUrl;

        // get photo info
        int infoStart = source.indexOf("<span class=\"quotation\">\"</span>") + (int)strlen("<span class=\"quotation\">\"</span>");
        int infoEnd = source.indexOf("</div>", infoStart);

        QString info = source.mid(infoStart, infoEnd - infoStart).trimmed();

        // get phofo title
        int titleStart = source.indexOf("<h2>", infoEnd) + 4;
        int titleEnd = source.indexOf("</h2>", titleStart);

        m_title = source.mid(titleStart, titleEnd - titleStart);

        // get photographer
        int phStart = source.indexOf("<b>", titleEnd) + 3;
        int phEnd = source.indexOf("</b>", phStart);

        QString photographer = source.mid(phStart, phEnd - phStart);
        // strip html tags
        photographer.replace(QRegExp("<[^>]*>"), "");

        m_info = QString("Photo by %1<br>%2").arg(photographer, info);

        KIO::StoredTransferJob *job = KIO::storedGet(m_OldUrl, KIO::Reload, KIO::HideProgressInfo);
        connect(job, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        // photo is the same as old one
        setPhotoSourceData();
        forceImmediateUpdateOfAllVisualizations();
    }
}

void StevesDigicams::photoDownloaded(KJob *job)
{
    if( job->error() )
    {
        setError( QString("[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ...").arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);


        if( m_photo.loadFromData(storedJob->data()) )
        {
            // everything is ok
            setPhotoSourceData();
            scheduleSourcesUpdated();
            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load pohto. Loading from cashe ..." );
            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(StevesDigicamsPotd, StevesDigicams)

#include "StevesDigicams.moc"

